@@ -5,10 +5,11 @@ opts_chunk$set(fig.path = 'figure/pos-', fig.align = 'center', fig.show = 'hold'
 options(warnPartialMatchArgs = FALSE)
 
 ## ------------------------------------------------------------------------
-library(ggplot2)
 library(ggpmisc)
+library(ggplot2)
 library(xts)
 library(lubridate)
+library(tibble)
 
 ## ------------------------------------------------------------------------
 class(austres)
@@ -217,8 +218,7 @@ ggplot(my.data, aes(x, y2)) +
   geom_point() +
   geom_smooth(method = "lm", formula = formula) +
   stat_poly_eq(aes(label = ..eq.label..), size = rel(3),
-               formula = formula, parse = TRUE,
-               label.x = 0, label.y = 2e6) +
+               formula = formula, parse = TRUE) +
   facet_wrap(~group)
 
 ## ------------------------------------------------------------------------
@@ -236,8 +236,7 @@ ggplot(my.data, aes(x, y2, colour = group)) +
   geom_point() +
   geom_smooth(method = "lm", formula = formula) +
   stat_poly_eq(aes(label = ..eq.label..),
-               formula = formula, parse = TRUE,
-               label.x = 0, label.y = c(1.8e6, 2e6)) +
+               formula = formula, parse = TRUE) +
   theme_bw()
 
 ## ------------------------------------------------------------------------
@@ -246,9 +245,27 @@ ggplot(my.data, aes(x, y2, colour = group)) +
   geom_point() +
   geom_smooth(method = "lm", formula = formula) +
   stat_poly_eq(aes(label = ..eq.label..),
-               formula = formula, parse = TRUE,
-               label.x = 0, label.y = 2e6,
-               vjust = c(1.2, 0)) +
+               formula = formula, parse = TRUE, label.y.npc = "center") +
+  theme_bw()
+
+## ------------------------------------------------------------------------
+formula <- y ~ poly(x, 3, raw = TRUE)
+ggplot(my.data, aes(x, y2, colour = group)) +
+  geom_point() +
+  geom_smooth(method = "lm", formula = formula) +
+  stat_poly_eq(aes(label = ..eq.label..),
+               formula = formula, parse = TRUE, label.y.npc = 0.75) +
+  theme_bw()
+
+## ------------------------------------------------------------------------
+formula <- y ~ poly(x, 3, raw = TRUE)
+ggplot(my.data, aes(x, y2, fill = block)) +
+  geom_point(shape = 21, size = rel(3)) +
+  geom_smooth(method = "lm", formula = formula) +
+  stat_poly_eq(aes(label = ..rr.label..), size = rel(3),
+               geom = "label", alpha = 0.33,
+               formula = formula, parse = TRUE) +
+  facet_wrap(~group, scales = "free_y") +
   theme_bw()
 
 ## ------------------------------------------------------------------------
@@ -259,23 +276,104 @@ ggplot(my.data, aes(x, y2, colour = group, fill = block)) +
   stat_poly_eq(aes(label = ..rr.label..), size = rel(3),
                geom = "label", alpha = 0.2,
                formula = formula, parse = TRUE,
-               label.x = 0, label.y = c(5e5, 5e5, 2e6, 2e6),
-               vjust = c(1.2,0,1.2,0)) +
+               label.y.npc = 0.66) +
   facet_wrap(~group, scales = "free_y") +
   theme_bw()
 
 ## ------------------------------------------------------------------------
 formula <- y ~ poly(x, 3, raw = TRUE)
-ggplot(my.data, aes(x, y2, fill = block)) +
-  geom_point(shape = 21, size = rel(3)) +
+ggplot(my.data, aes(x, y)) +
+  geom_hline(yintercept = 0, linetype = "dashed") +
+  stat_fit_residuals(formula = formula)
+
+## ------------------------------------------------------------------------
+formula <- y ~ poly(x, 3, raw = TRUE)
+ggplot(my.data, aes(x, y)) +
+  geom_hline(yintercept = 0, linetype = "dashed") +
+  stat_fit_residuals(formula = formula, resid.type = "working")
+
+## ------------------------------------------------------------------------
+formula <- y ~ poly(x, 3, raw = TRUE)
+ggplot(my.data, aes(x, y, color = group)) +
+  geom_hline(yintercept = 0, linetype = "dashed") +
+  stat_fit_residuals(formula = formula)
+
+## ------------------------------------------------------------------------
+formula <- y ~ poly(x, 3, raw = TRUE)
+ggplot(my.data, aes(x, y)) +
   geom_smooth(method = "lm", formula = formula) +
-  stat_poly_eq(aes(label = ..rr.label..), size = rel(3),
-               geom = "label", alpha = 0.33,
-               formula = formula, parse = TRUE,
-               label.x = 0,
-               vjust = c(1.2,0,1.2,0)) +
-  facet_wrap(~group, scales = "free_y") +
-  theme_bw()
+  stat_fit_deviations(formula = formula, color = "red") +
+  geom_point()
+
+## ------------------------------------------------------------------------
+formula <- y ~ poly(x, 3, raw = TRUE)
+ggplot(my.data, aes(x, y, color = group)) +
+  geom_smooth(method = "lm", formula = formula) +
+  stat_fit_deviations(formula = formula) +
+  geom_point()
+
+## ------------------------------------------------------------------------
+formula <- y ~ poly(x, 3, raw = TRUE)
+ggplot(my.data, aes(x, y)) +
+  geom_smooth(method = "lm", formula = formula) +
+  stat_fit_deviations(formula = formula, color = "red",
+                      arrow = arrow(length = unit(0.015, "npc"), 
+                                   ends = "both")) +
+  geom_point()
+
+## ------------------------------------------------------------------------
+# formula <- y ~ poly(x, 3, raw = TRUE)
+# broom::augment does not handle poly correctly!
+formula <- y ~ x + I(x^2) + I(x^3)
+ggplot(my.data, aes(x, y)) +
+  geom_point() +
+  geom_smooth(method = "lm", formula = formula) +
+  stat_fit_glance(method = "lm", 
+                  method.args = list(formula = formula),
+                  geom = "text",
+                  aes(label = signif(..p.value.., digits = 4)))
+
+## ------------------------------------------------------------------------
+# formula <- y ~ poly(x, 3, raw = TRUE)
+# broom::augment does not handle poly() correctly!
+formula <- y ~ x + I(x^2) + I(x^3)
+ggplot(my.data, aes(x, y, color = group)) +
+  geom_point() +
+  geom_smooth(method = "lm", formula = formula) +
+  stat_fit_glance(method = "lm", 
+                  method.args = list(formula = formula),
+                  geom = "text", 
+                  aes(label = paste("P-value = ", signif(..p.value.., digits = 4), sep = "")))
+
+## ------------------------------------------------------------------------
+# formula <- y ~ poly(x, 3, raw = TRUE)
+# broom::augment does not handle poly correctly!
+formula <- y ~ x + I(x^2) + I(x^3)
+ggplot(my.data, aes(x, y, color = group)) +
+  geom_point() +
+  geom_smooth(method = "lm", formula = formula) +
+  stat_fit_glance(method = "lm", 
+                  method.args = list(formula = formula),
+                  label.x.npc = "right",
+                  label.y.npc = "bottom",
+                  geom = "text", 
+                  aes(label = paste("P-value = ", signif(..p.value.., digits = 4), sep = "")))
+
+## ------------------------------------------------------------------------
+# formula <- y ~ poly(x, 3, raw = TRUE)
+# broom::augment does not handle poly correctly!
+formula <- y ~ x + I(x^2) + I(x^3)
+ggplot(my.data, aes(x, y)) +
+  geom_point() +
+  stat_fit_augment(method = "lm",
+                   method.args = list(formula = formula))
+
+## ------------------------------------------------------------------------
+formula <- y ~ x + I(x^2) + I(x^3)
+ggplot(my.data, aes(x, y, color = group)) +
+  geom_point() +
+  stat_fit_augment(method = "lm", 
+                   method.args = list(formula = formula))
 
 ## ------------------------------------------------------------------------
 ggplot(my.data, aes(x, y)) + stat_debug_group()
@@ -316,7 +414,7 @@ ggplot(my.data, aes(x, y)) + geom_point() +
 
 ## ------------------------------------------------------------------------
 ggplot(my.data, aes(x, y)) + geom_point() + 
-  stat_debug_group(summary.fun = dplyr::as_data_frame)
+  stat_debug_group(summary.fun = as_data_frame)
 
 ## ------------------------------------------------------------------------
 ggplot(my.data, aes(x, y)) + geom_point() + 
@@ -344,13 +442,47 @@ ggplot(my.data, aes(x, y, colour = group)) + geom_point() +
 ggplot(my.data, aes(x, y, colour = group)) + geom_point() + 
   geom_debug(summary.fun = head)
 
+## ------------------------------------------------------------------------
+ggplot(my.data, aes(x, y, colour = group)) + geom_point() + 
+  stat_smooth(method = "lm",
+             geom = "debug", 
+             summary.fun = as_data_frame, 
+             summary.fun.args = list())
+
 ## ------------------------------------------------------------------------
 ggplot(my.data, aes(x, y, colour = group)) + geom_point() + 
   stat_peaks(span = NULL,
              geom = "debug", 
-             summary.fun = function(x) {x}, 
+             summary.fun = as_data_frame, 
              summary.fun.args = list())
 
+## ------------------------------------------------------------------------
+formula <- y ~ poly(x, 3, raw = TRUE)
+ggplot(my.data, aes(x, y)) +
+  stat_fit_residuals(formula = formula, 
+                     geom = "debug",
+                     summary.fun = as_data_frame, 
+                     summary.fun.args = list())
+
+## ------------------------------------------------------------------------
+formula <- y ~ poly(x, 3, raw = TRUE)
+ggplot(my.data, aes(x, y, color = group)) +
+  geom_point() +
+  stat_fit_augment(method = "lm", 
+                   method.args = list(formula = formula),
+                   geom = "debug",
+                     summary.fun = as_data_frame, 
+                     summary.fun.args = list(),
+                   aes(y = ...fitted..,
+                       ymax = ...fitted.. + ...se.fit.. * 2,
+                       ymin = ...fitted.. - ...se.fit.. * 2))+
+  stat_fit_augment(method = "lm", 
+                   method.args = list(formula = formula),
+                   geom = "smooth",
+                   aes(y = ...fitted..,
+                       ymax = ...fitted.. + ...se.fit.. * 2,
+                       ymin = ...fitted.. - ...se.fit.. * 2))
+
 ## ------------------------------------------------------------------------
 ggplot(my.data, aes(x, y, colour = group)) + geom_null()
 
